(vl-load-com)

(defun c:pltable (
                 /
                 app doc model pt
                 ent ent_n ent_o coords len
                 nrows ncols rh cw tbl
                 idx x y i
                 blk blk_n blkid attid
                 )

  (setq app (vlax-get-acad-object)
        doc (vla-get-ActiveDocument app)
        model (vla-get-ModelSpace doc)
        pt (vlax-3d-point 0 0 0)
  )
  
  (setq ent (ssget "+.:E:S" '((0 . "LWPOLYLINE"))))
  
  (if ent
    (progn
      (setq ent_n (ssname ent 0)
            ent_o (vlax-ename->vla-object ent_n)
            coords (vla-get-coordinates ent_o)
            coords (vlax-safearray->list (vlax-variant-value coords))
            len (length coords)
      )
      
      ; add table
      (setq nrows (+ 2 (/ len 2))
            ncols 3
            rh 10 ; row height
            cw 30 ; column width
            tbl (vla-Addtable model pt nrows ncols 10 30)
      )
      
      ; add title and header
      (vla-setcellvalue tbl 0 0 "Coordinates")
      (vla-setcellvalue tbl 1 0 "Vertex")
      (vla-setcellvalue tbl 1 1 "X")
      (vla-setcellvalue tbl 1 2 "Y")
      
      
      ; loop through vertices
      (setq i 0)
      (while (< i len)
        (setq idx (+ 1 (/ i 2))
              x (nth i coords)
              y (nth (+ 1 i) coords)
        )
        
        ; add block
        (setq inspt (vlax-3d-point x y 0))
        (setq blk (vla-insertblock model inspt "Pos" 1 1 1 0)
              blkid (vla-get-objectid blk)
              blk_n (vlax-vla-object->ename blk)
              attid (kdub:getAttEntID blk_n "PTNR")
        )
        
        ; convert id-s to strings
        (setq blkid (vl-princ-to-string blkid)
              attid (vl-princ-to-string attid)
        )
        
        ; set attribute
        (LM:vl-setattributevalue blk "PTNR" (itoa idx))
        
        ; generate field strings
        (setq xfield (strcat 
                       "%<\\AcObjProp Object(%<\\_ObjId "
                       blkid
                       ">%).InsertionPoint \\f \"%lu6%pt1\">%")
              
              yfield (strcat 
                       "%<\\AcObjProp Object(%<\\_ObjId "
                       blkid
                       ">%).InsertionPoint \\f \"%lu6%pt2\">%")
              
              idxfield (strcat 
                         "%<\\AcObjProp Object(%<\\_ObjId "
                         attid
                        ">%).TextString>%")
        )
        
        ; fill in the table
        (vla-settext tbl (+ idx 1) 0 idxfield)
        (vla-settext tbl (+ idx 1) 1 xfield)
        (vla-settext tbl (+ idx 1) 2 yfield)
        
        (setq i ( + 2 i))
      )
    )
  )

  (princ)

)




;; Set Attribute Value  -  Lee Mac
;; Sets the value of the first attribute with the given tag found within the block, if present.
;; blk - [vla] VLA Block Reference Object
;; tag - [str] Attribute TagString
;; val - [str] Attribute Value
;; Returns: [str] Attribute value if successful, else nil.

(defun LM:vl-setattributevalue ( blk tag val )
    (setq tag (strcase tag))
    (vl-some
       '(lambda ( att )
            (if (= tag (strcase (vla-get-tagstring att)))
                (progn (vla-put-textstring att val) val)
            )
        )
        (vlax-invoke blk 'getattributes)
    )
)



; this is from the Internet
(defun kdub:getAttEntID ( ent tag / data )
    ;;CodeHimBelonga kdub
  (while (and (setq ent (entnext ent))
              (setq data (entget ent))
              (eq (cdr (assoc 0 data)) "ATTRIB")
              (/= (strcase (cdr (assoc 2 data))) tag)
         )
  )
  (if (eq (cdr (assoc 0 data)) "ATTRIB")
    (vla-get-ObjectID (vlax-ename->vla-object ent))
  )
)
